package com.myoa.engineering.sample.configuration.logging;

import jakarta.servlet.ReadListener;
import jakarta.servlet.ServletInputStream;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletRequestWrapper;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

public class RereadableRequestWrapper extends HttpServletRequestWrapper {

    private final ByteArrayOutputStream contents = new ByteArrayOutputStream();

    public RereadableRequestWrapper(HttpServletRequest request) {
        super(request);
    }

    @Override
    public ServletInputStream getInputStream() throws IOException {
        super.getInputStream().transferTo(contents);
        return new RereadableServletInputStream(contents);
    }

    public byte[] getContents() throws IOException {
        return this.getInputStream().readAllBytes();
    }

    private static class RereadableServletInputStream extends ServletInputStream {

        private final ByteArrayInputStream buffer;

        public RereadableServletInputStream(final ByteArrayOutputStream contents) {
            buffer = new ByteArrayInputStream(contents.toByteArray());
        }

        @Override
        public boolean isFinished() {
            return buffer.available() == 0;
        }

        @Override
        public boolean isReady() {
            return true;
        }

        @Override
        public void setReadListener(ReadListener listener) {
            throw new RuntimeException("Not implemented");
        }

        @Override
        public int read() throws IOException {
            return buffer.read();
        }
    }

}
